#include "3dobjects.h"
#include "mesh3ds.h"	
#include "water.h"

/************************************************************************
*	class c3DObject implementation										*
*	first class to represent 3d objects (single o3d mesh)				*
************************************************************************/
//	default constructor.. constructs an abstract mesh
c3DObject::c3DObject()	:m_Loc(*(point3d*)&m_WorldMatrix._41)
{
	m_WorldMatrix.makeident();
	m_Mesh=new sMesh();
	SetMaterial(sMaterial::BRUSHED_METAL);
}

//	draw using local matrix
void c3DObject::Draw()
{
	m_Material.Activate();									//	activate the material

	glPushMatrix();											//	save current modelview matrix
	glMultMatrixf(m_WorldMatrix.d);							//	multiply by the world matrix 

	if(m_Mesh->m_Texture)
	{
		m_Mesh->m_Texture->Activate();
		m_Mesh->Draw();	
		m_Mesh->m_Texture->Deactivate();
	}
	else
		m_Mesh->Draw();	

	glPopMatrix();
}

// use this to scale the model..
void c3DObject::Scale(float amt)
{
	matrix4 scalemat;
	scalemat.makeident();
	scalemat._11=scalemat._22=scalemat._33=amt;
	m_WorldMatrix=scalemat*m_WorldMatrix;
	m_Mesh->ScaleNormals(amt);
}


// use this to scale the model..
void c3DObject::Scale(float sx,float sy,float sz)
{
	matrix4 scalemat;
	scalemat.makeident();
	scalemat._11=sx;
	scalemat._22=sy;
	scalemat._33=sz;
	m_WorldMatrix=scalemat*m_WorldMatrix;
	m_Mesh->ScaleNormals(sx,sy,sz);
}


//	show local object i,j,k
void c3DObject::ShowAxis()
{
	glPushMatrix();
	glMultMatrixf(m_WorldMatrix.d);

	glPushAttrib(GL_LIGHTING_BIT);
	glDisable(GL_LIGHTING);

	glBegin(GL_LINES);
	
		glColor3f(1,0,0);
		glVertex3f(0,0,0);
		glVertex3f(1,0,0);
		glColor3f(0,1,0);
		glVertex3f(0,0,0);
		glVertex3f(0,1,0);
		glColor3f(0,0,1);
		glVertex3f(0,0,0);
		glVertex3f(0,0,1);

	glEnd();

	glPopMatrix();
	glPopAttrib();
}


//	make athe mesh as a sphere
void c3DObject::CreateSphere(float radius,int numSphereRings,int numSphereSegments)
{
	SAFE_DELETE(m_Mesh);
	m_Mesh=new sStridedMesh;	
	((sStridedMesh*)m_Mesh)->CreateSphere(radius,numSphereRings,numSphereSegments);
}

//	make the mesh as a disk
void c3DObject::CreateDisk(float InnerRadius,float OuterRadius)
{
	SAFE_DELETE(m_Mesh);
	m_Mesh=new sDiskMesh(InnerRadius,OuterRadius);
}

//	create a cylinder
void c3DObject::CreateCylinder(float baseradius,float topradius, float height)
{
	SAFE_DELETE(m_Mesh);
	m_Mesh=new sCylinderMesh(baseradius,topradius,height);
}


//	generate a box and send the nuber of sections required
void c3DObject::CreateBox(float x,float y,float z)
{
	SAFE_DELETE(m_Mesh);
	m_Mesh= new sIndexedMesh();
	((sIndexedMesh*)m_Mesh)->CreateBox(x,y,z,10,10,10,6);
}

//	generate an object from a file..
//	supported extesnions are o3d,h3d,3ds
void c3DObject::CreateObject(char* filename)
{
	//	extension of the 3d file
	char *extStr;

	// get extension from filename
	extStr = strchr(&filename[1], '.');
	extStr++;

	//	first delete the mesh
	SAFE_DELETE(m_Mesh);

	//	set the file type according to the extensions sent
	if ((strcmp(extStr, "o3d") == 0) || (strcmp(extStr, "O3D") == 0)||(strcmp(extStr, "o3dt") == 0)||(strcmp(extStr, "O3DT") == 0))
		m_Mesh=new sIndexedMesh(filename);
	else if((strcmp(extStr, "h3d") == 0) || (strcmp(extStr, "H3D") == 0)||(strcmp(extStr, "h3dt") == 0)||(strcmp(extStr, "H3DT") == 0))
		m_Mesh=new sHierMesh(filename);
	else if((strcmp(extStr, "3ds") == 0) || (strcmp(extStr, "3DS") == 0))
	{
		m_Mesh	=	 new s3dsMesh();
		((s3dsMesh*)m_Mesh)->Load3DS(filename);
	}
	else
		Application()->ReportError("3D Object file format %s still supported",extStr);
}


//	creates an object as a text object
void c3DObject::CreateText(char * String)
{
	SAFE_DELETE(m_Mesh);
	m_Mesh	=	 new sTextMesh(String);

}

//	makes the mesh a water mesh...
void c3DObject::CreateWater(float Width,float Height,int numVertsx,int numVertsy)
{
	SAFE_DELETE(m_Mesh);
	m_Mesh=new sWaterMesh(Width,Height,numVertsx,numVertsy);	
}

//	create a water container object
void c3DObject::CreateWaterContainer(int NumRows,int NumColumns,float Depth)
{
	SAFE_DELETE(m_Mesh);
	SetMaterial(sMaterial::OPENGL_DEFAULT);
	m_Mesh=new sRealWater(NumRows,NumColumns,Depth,0.02f);
}

//	makes the mesh a water mesh...
void c3DObject::CreateTorus(float R1,float R2)
{
	SAFE_DELETE(m_Mesh);
	m_Mesh=new sTorusMesh(R1,R2);	
}


//	makes the mesh a Terrain mesh...
void c3DObject::CreateTerrain(float Width,float Depth,char* MapFileName,float HeightScale)
{
	SAFE_DELETE(m_Mesh);
	m_Mesh=new sTerrainMesh(Width,Depth,MapFileName,HeightScale);	
}


//	crete stars
void c3DObject::CreateStars(int NumStars)
{
	SAFE_DELETE(m_Mesh);
	m_Mesh	=	new sStarsMesh(NumStars);
}





// this will translate the object relative to its parent or to the world if root
void c3DObject::Translate(float x,float y,float z)
{
	m_WorldMatrix._41+=x;
	m_WorldMatrix._42+=y;
	m_WorldMatrix._43+=z;
}

// this will translate the object relative to its local axis not world axis
void c3DObject::TranslateLocal(float x,float y,float z)
{
	matrix4 move;
	point3d step(x,y,z);
	move.istranslation(step);
	MatMult(move,m_WorldMatrix,&m_WorldMatrix);
	m_WorldMatrix=move*m_WorldMatrix;
}

// this will rotate the matrix on x,y,z axis parralel to those of the world
void c3DObject::Rotate(float x,float y,float z)
{
	m_WorldMatrix.rotate(x,y,z);
}

//	rotation in object space
void c3DObject::RotateLocal(float x,float y,float z)
{
	matrix4 rot;
	rot.makeident();
	rot.rotate(x,y,z);
	m_WorldMatrix=rot*m_WorldMatrix;
}


//	set the position
void c3DObject::SetPos(float x,float y, float z)
{
	m_WorldMatrix._41=x;
	m_WorldMatrix._42=y;
	m_WorldMatrix._43=z;
}

/*-------------end of class c3DObject implementation---------------*/


/********************************************************************
*	class c3DComplexObject implementation							*
*	second level class. hierarchical objects						*
********************************************************************/
//	Draw the whole object and its children
void c3DComplexObject::DrawTree()
{
	m_Material.Activate();					//	activate the material
	glPushMatrix();
	glMultMatrixf(m_WorldMatrix.d);			//	multiply by the world matrix 

	if(m_Mesh->m_Texture)
	{
		m_Mesh->m_Texture->Activate();
		m_Mesh->Draw();	
		m_Mesh->m_Texture->Deactivate();
	}
	else
		m_Mesh->Draw();	


	for(int i=0;i<m_Childs.size();i++)		//	loop on all childs
		m_Childs[i]->Draw();				//	and recurse
	glPopMatrix();
}

// link the hmodel to the other as its child at the location desired relative
// to the parent
void c3DComplexObject::LinkTo(c3DComplexObject& parent,point3d& at,point3d& rot)
{
	//	set out parent to be that object
	m_pParentObject	=	&parent;
	// set this object in its parent data
	parent.m_Childs.push_back(this);
	// and let the matrix have the location sent
	//m_worldmatrix.rotate(rot.x,rot.y,rot.z);
	m_WorldMatrix.place(at);
}
/*--------end of class c3DComplexObject implementation-------*/



