#ifndef _3DOBJECTS_H
#define _3DOBJECTS_H

/****************************************************************
*	3dobjects.h													*
*	By Shady El Mously											*
*	part of the flulib engin									*
*	this is th file which contains the classes represting the	*
*	3d object that could appear in the application				*
****************************************************************/
#include "3dtypes.h"
#include "meshes.h"



/************************************************************************
*	class c3DObject														*
*	By	Shady El Mously													*
*	this class is the base class to represent							*
*	a	mesh	of points representing a 3d object						*
*	it uses a file named .o3d for input									*
*	o3d files can be text or binary										*
*	each cmodel has an associated matrix, the world matrix				*
*	u can send a matrix to draw also..									*
*	each object is mainly a big array of vertices						*
*	and another arrays of indexes										*
************************************************************************/
class c3DObject
{
protected:

public:

	matrix4			m_WorldMatrix;	//	matrix of the object world transform
	sMesh *			m_Mesh;			//	virtual mesh to speed up calls using vfunctions
	sMaterial		m_Material;		//	material used, protected for inheritance
	point3d&		m_Loc;			//	posiiton of the object
	

	//	default constructor.. constructs an abstract mesh
	c3DObject();

	virtual ~c3DObject()
	{
		SAFE_DELETE(m_Mesh);
	}

	//	external use of the world matrix here
	matrix4& GetWorldMatrix()
	{
		return m_WorldMatrix;
	}

	//	set a material to the object
	void SetMaterial(const sMaterial& Material){m_Material=Material;}

	//	adds a texture to the object if it is one of the simple ones
	void SetTexture(char* FileName)
	{
		SAFE_DELETE (m_Mesh->m_Texture);
		m_Mesh->m_Texture=new cTexture(FileName);
	}


	// use this to scale the model..
	virtual void Scale(float amt);
	// use this to scale the model in 3 directions
	virtual void Scale(float sx,float sy,float sz);

	//	draw using local matrix
	virtual void Draw();
	
	//	will show the object world axis...it is not a 3d function
	virtual void ShowAxis(void);

	//	this will generate a strided mesh sphere...
	virtual void CreateSphere(float radius,int numSphereRings=30,int numSphereSegments=30);

	//	generate a box and send the nuber of sections required
	virtual void CreateBox(float x,float y,float z);

	//	generate an object from a file..
	//	supported extesnions are o3d,h3d,3ds
	virtual void CreateObject(char* filename);

	//	create a disk
	virtual void CreateDisk(float InnerRadius,float OuterRadius);

	//	create a cylinder
	virtual void CreateCylinder(float baseradius,float topradius, float height);

	//	makes the mesh a water mesh...
	virtual void CreateWater(float Width,float Height,int numVertsx,int numVertsy);

	//	makes the mesh a Terrain mesh...
	virtual void CreateTerrain(float Width,float Depth,char* MapFileName,float HeightScale);

	//	create a torus
	virtual void CreateTorus(float R1,float R2);
	
	//	creates a cloud of stars
	virtual void CreateStars(int NumStars);

	//	creates an object as a text object
	virtual void CreateText(char * String);

	//	create a water container object
	virtual void CreateWaterContainer(int NumRows,int NumColumns,float Depth);



	//	position and orientation control functions
	
	// this will rotate the matrix on x,y,z axis parralel to those of the world
	virtual void Rotate(float x,float y,float z);

	//	rotation in object space around the object space axis
	virtual void RotateLocal(float x,float y,float z);

	// this will translate the object relative to its parent or to the world if root
	virtual void Translate(float x,float y,float z);

	// this will translate the object relative to its local axis not world axis
	virtual void TranslateLocal(float x,float y,float z);

	//	set the position
	virtual void SetPos(float x,float y, float z);

};
/*---------end of class c3DObject declaration----------------------*/



/********************************************************************
*	class c3DComplexObject											*
*	this class is the second class for representing 3d objects		*
*	the first was cmodel											*
*	this class creates 3d objects as hierachical objects			*
*	this is very useful in forward inverse kinematics				*
*	objects from this type may be linked together					*
*	the parent object will draw all its children					*
*	relative to it.. all objects have a matrix relative to their	*
*	parents this matrix repreent a translation						*
*	a fixed point in their parent place and rotation around the		*
*	axes at that point												*
*	this is the only class that used the xfiles						*
*	the code for the xfile is commented to be used only when needed	*
********************************************************************/

class c3DComplexObject : public c3DObject
{
private:

	vector <c3DComplexObject*>	m_Childs;			//	vector of children hierachical objects
	c3DComplexObject*			m_pParentObject;	//	parent object to which this object is linked

public:


	//	constructor
	c3DComplexObject()			{m_pParentObject	=	NULL;}
	
	//	destructor free the pointers
	~c3DComplexObject(){}

	//	link the hmodel to the other as its child at the location desired relative to parent
	void LinkTo(c3DComplexObject& parent,point3d & at,point3d & rot=point3d(0,0,0));

	//	Draw the object and all its children
	void DrawTree();

};
/*-------------end of class c3DComplexObject declaration--------------*/

#endif // _3DOBJECTS_H