/********************************************************************
*	implementation of all 3d classes								*
*	textures, models, lights, materials, cameras classes			*
*	are all implemented here										*
********************************************************************/

#include	"3dtypes.h"


using namespace std;




/********************************************************************
*	class light	implementation										*
*	By Shady el Mously												*
********************************************************************/

//	base fo lights
int sLight::m_Base=0;

// default constructor.. just increment number of lights used in the scene

sLight::sLight	(
				const color3&	diff,
				const color3&	spec,
				const color3&	amb
				):			
				dcvDiffuse		(diff),dcvSpecular(spec),dcvAmbient(amb)
{
		if(m_Base>7)
			Application()->ReportError("Cant create more than 8 lights!");

		m_GLid	=	m_Base;
		m_Base++;
}

//	normal activation for any light
//	setting the light's colors
void sLight::Activate()
{
	//	enable the  light
	glEnable(GL_LIGHT0+m_GLid);	

	//	set the light colors
	glLightfv(GL_LIGHT0+m_GLid,GL_AMBIENT	,(const float *)&dcvAmbient	);
	glLightfv(GL_LIGHT0+m_GLid,GL_DIFFUSE	,(const float *)&dcvDiffuse	);
	glLightfv(GL_LIGHT0+m_GLid,GL_SPECULAR	,(const float *)&dcvSpecular);
}
/*--------------end of struct sLight implementation-----------------*/



/********************************************************************
*	struct sDirectionalLight implementation							*	
*	By Shady el Mously												*
********************************************************************/

// construct a directional light
sDirectionalLight::sDirectionalLight(
	const point3d& dir,
	const color3&	diff,
	const color3&	spec,
	const color3&	amb): sLight(diff,spec,amb)
{
	//	since this is directional.. fourth component in the light position should be 0.0
	dvPosition=point3d(dir.x,dir.y,dir.z);
	dvPosition.w=0.0f;				//	zero mean it is a directional light
}

//	activation fo directional light
void sDirectionalLight::Activate()
{
	//	do the basic work
	sLight::Activate();

	//	and set the direction of the directional light
	glLightfv(GL_LIGHT0+m_GLid,GL_POSITION,dvPosition.v);
}
/*-----------end of struct sDirectionalLight implementation------------*/



/************************************************************************
*	struct sPointLight implementation									*
*	By Shady El Mously													*
************************************************************************/

// construct a point source light
sPointLight::sPointLight(		
	const point3d& loc,
	const color3&	diff,
	const color3&	spec,
	const color3&	amb) : sLight(diff,spec,amb)
{
	// make the light ready and fill it
	dvRange			=	SLIGHT_RANGE_MAX;
	dvPosition		=	point3d(loc.x,loc.y,loc.z);

	dvPosition.w=1.0f;		//	1 means a point source
}


//	activate a point light
void sPointLight::Activate()
{
	//	do the basic work
	sLight::Activate();

	//	and set its position
	glLightfv(GL_LIGHT0+m_GLid,GL_POSITION,dvPosition.v);
}

/*----------------end of struct sPointLight implementation-------------*/


/************************************************************************
*	struct sSpotLight implementation									*
*	By Shady El Mously													*
************************************************************************/
// construct a spotligh
sSpotLight::sSpotLight(
	const point3d& loc,
	const point3d& dir,
	float theta,
	const color3&	diff,
	const color3&	spec,
	const color3&	amb): sLight(diff,spec,amb)
{
	dvRange			=	SLIGHT_RANGE_MAX;
	dvTheta			=	theta;
	dvPosition		=	point3d(loc.x,loc.y,loc.z);
	dvDirection		=	dir;

	dvPosition.w=1.0f;		//	1.0 for spot lights too

}

// turn on the light
void sSpotLight::Activate()
{
	//	do the basic work
	sLight::Activate();

	//	set the spot light specific parameters..direction and theta
	glLightfv(GL_LIGHT0+m_GLid,GL_SPOT_DIRECTION,(const float *)&dvDirection);
	glLightfv(GL_LIGHT0+m_GLid,GL_SPOT_CUTOFF,(const float *)&dvTheta);

	//	set the spot position
	glLightfv(GL_LIGHT0+m_GLid,GL_POSITION,dvPosition.v);
}
/*-------------end of class sSpotLight implementation--------------*/



/********************************************************************
*	class material	implementation									*
*	By Shady el Mously												*
********************************************************************/


//	a white material
//	changing the material
void sMaterial::Set(
	color4& amb,
	color4& diff,
	color4& spec,
	color4& emit,
	float pow)
{
	ZeroMemory(this,sizeof(sMaterial));
	diffuse		=	diff;
	ambient		=	amb;
	specular	=	spec;
	emissive	=	emit;
	power		=	pow;
}

// make the material the active one
void sMaterial::Activate()
{
	m_bIsActive	=	true;
	glPushAttrib(GL_LIGHTING_BIT);			//	store previous material
	glMaterialfv(GL_FRONT,GL_SPECULAR	,	(const float *)&specular	);
	glMaterialfv(GL_FRONT,GL_DIFFUSE	,	(const float *)&diffuse	);
	glMaterialfv(GL_FRONT,GL_AMBIENT	,	(const float *)&ambient	);
	glMaterialfv(GL_FRONT,GL_EMISSION	,	(const float *)&emissive	);
	glMaterialfv(GL_FRONT,GL_SHININESS	,	(const float *)&power		);
}


// some standard materials from 3d studio

const sMaterial sMaterial::AQUA_GLAZE
(RGBCOLOR(0,42,41,127),RGBCOLOR(0,113,109,127),RGBCOLOR(253,254,254,127));

const sMaterial sMaterial::AQUA_GLAZE_METAL
(RGBCOLOR(0,42,41,127),RGBCOLOR(0,113,109,127),RGBCOLOR(255,255,255,127));

const sMaterial sMaterial::BARREL_BLUE
(RGBCOLOR(15,13,0,127),RGBCOLOR(92,101,216,127),RGBCOLOR(0,0,0,127));

const sMaterial sMaterial::BEIGE_MATTE
(RGBCOLOR(67,53,32,127),RGBCOLOR(156,123,75,127),RGBCOLOR(206,207,159,127));

const sMaterial sMaterial::BEIGE_PLASTIC
(RGBCOLOR(67,53,32,127),RGBCOLOR(156,123,75,127),RGBCOLOR(254,254,253,127));

const sMaterial sMaterial::BLACK_MATTE
(RGBCOLOR(1,1,1,127),RGBCOLOR(6,6,6,127),RGBCOLOR(122,122,122,127));

const sMaterial sMaterial::BLACK_PLASTIC
(RGBCOLOR(1,1,1,127),RGBCOLOR(3,3,3,127),RGBCOLOR(255,255,255,127));

const sMaterial sMaterial::BLUE_BUMPY_CLOTH
(RGBCOLOR(1,2,7,127),RGBCOLOR(14,26,72,127),RGBCOLOR(116,116,154,127));

const sMaterial sMaterial::BLUE_METAL_SKY
(RGBCOLOR(0,0,7,127),RGBCOLOR(23,32,214,127),RGBCOLOR(255,255,255,127));

const sMaterial sMaterial::BLUE_PLASTIC
(RGBCOLOR(0,0,93,127),RGBCOLOR(0,0,254,127),RGBCOLOR(253,253,255,127));

const sMaterial sMaterial::BRASS
(RGBCOLOR(5,3,0,127),RGBCOLOR(165,99,62,127),RGBCOLOR(255,255,255,127));

const sMaterial sMaterial::BRUSHED_METAL
(RGBCOLOR(0,0,0,127),RGBCOLOR(255,255,255,127),RGBCOLOR(127,127,127,127));

const sMaterial sMaterial::CEILING_TILE
(RGBCOLOR(117,116,110,127),RGBCOLOR(234,233,223,127),RGBCOLOR(252,251,240,127));

const sMaterial sMaterial::CHERRY_RED_METAL
(RGBCOLOR(5,0,0,127),RGBCOLOR(142,6,4,127),RGBCOLOR(255,0,0,127));

const sMaterial sMaterial::CHROME_BLUE_SKY
(RGBCOLOR(1,6,7,127),RGBCOLOR(196,227,255,127),RGBCOLOR(255,255,127,127));

const sMaterial sMaterial::CHROME_MOROCO
(RGBCOLOR(8,5,0,127),RGBCOLOR(228,209,185,127),RGBCOLOR(255,127,127,127));

const sMaterial sMaterial::CHROME_PEARL
(RGBCOLOR(1,1,1,127),RGBCOLOR(217,134,183,127),RGBCOLOR(91,209,219,127));

const sMaterial sMaterial::SKY_BLUE_METAL
(RGBCOLOR(0,0,7,127),RGBCOLOR(23,32,214,127),RGBCOLOR(255,255,255,127));

const sMaterial sMaterial::OPENGL_DEFAULT
(color4(0.2f,0.2f,0.2f,1.0f),color4(0.8f,0.8f,0.8f,1.0f),color4(0.0f,0.0f,0.0f,1.0f),color4(0.0f,0.0f,0.0f,1.0f));


/*---------------end of struct sMaterial implementation-------------*/
	



/********************************************************************
*	class camera	implementation									*
*	By Shady el Mously												*
********************************************************************/


//	construct with a location and 3 euler angles
cCamera::cCamera(point3d& loc,float y,float p,float r):TimeDelta(FrameController()->m_DeltaTime)
{
	Init(loc,y,p,r);
}

//	initialize the camera
void cCamera::Init(point3d loc,float y,float p,float r)
{
	m_Loc	=	loc;
	m_Yaw	=	y;
	m_Pitch	=	p;
	m_Roll	=	r;
	m_Speed	=	0.01f;
	ComputeInfo();
}

//	this function should be called to setup all the internal data
//	it should be used after any rotate/transate functions before activating the camera
void cCamera::ComputeInfo()
{
	float CosY,CosP,CosR;
	float SinY,SinP,SinR;

	CosY	=	cosf(m_Yaw);
	CosP	=	cosf(m_Pitch);
	CosR	=	cosf(m_Roll);
	SinY	=	sinf(m_Yaw);
	SinP	=	sinf(m_Pitch);
	SinR	=	sinf(m_Roll);

	//	fwd vector
	m_Fwd.x=SinY*CosP;
	m_Fwd.y=SinP;
	m_Fwd.z=CosP*-CosY;

	//	look at point
	m_At=m_Fwd+m_Loc;

	//	up vector
	m_Up.x=-CosY*SinR-SinY*SinP*CosR;
	m_Up.y=CosP*CosR;
	m_Up.z=-SinY*SinR-SinP*CosR*-CosY;

	//	side vector :
	m_Side=m_Fwd^m_Up;
}
//	calling this will make the activate the camera
void cCamera::Activate()	
{
	glMatrixMode(GL_MODELVIEW);										//	set matrix mode to modelview
	glLoadIdentity();												//	load identity
	gluLookAt(m_Loc.x,m_Loc.y,m_Loc.z,m_At.x,m_At.y,m_At.z,m_Up.x,m_Up.y,m_Up.z);	//	set the camera orientation
}


// move the camera..relative to the world axis
void cCamera::Translate(float x,float y, float z)
{
	m_Loc.x+=x;
	m_Loc.y+=y;
	m_Loc.z+=z;
	ComputeInfo();
}


//	move camera reltive to viewer, left and right are like the strafe
void cCamera::TranslateLocal(float x,float y,float z)
{
	m_Loc += TimeDelta * z * m_Fwd;
	m_Loc += TimeDelta * x * m_Side;
	m_Loc += TimeDelta * y * m_Up;
	ComputeInfo();
}

//	rotate the camera around its own axis..
void cCamera::RotateLocal(float yaw,float pitch,float roll)
{
	m_Yaw		+=yaw;
	m_Pitch		+=pitch;
	m_Roll		+=roll;
	ComputeInfo();
}

//	control the camera with keyboard
void cCamera::KeyBoard() 
{
	// Keyboard Data
	if (KEYDOWN('W'		)) m_Loc += TimeDelta * m_Speed * m_Fwd;
	if (KEYDOWN('S'		)) m_Loc -= TimeDelta * m_Speed * m_Fwd;
	if (KEYDOWN('D'		)) m_Loc += TimeDelta * m_Speed * m_Side;
	if (KEYDOWN('A'		)) m_Loc -= TimeDelta * m_Speed * m_Side;
	if (KEYDOWN(VK_UP	)) m_Loc += TimeDelta * m_Speed * m_Fwd;
	if (KEYDOWN(VK_DOWN	)) m_Loc -= TimeDelta * m_Speed * m_Fwd;
	if (KEYDOWN(VK_RIGHT)) m_Loc += TimeDelta * m_Speed * m_Side;
	if (KEYDOWN(VK_LEFT	)) m_Loc -= TimeDelta * m_Speed * m_Side;
	if (KEYDOWN('R'		)) m_Loc += TimeDelta * m_Speed * m_Up;
	if (KEYDOWN('F'		)) m_Loc -= TimeDelta * m_Speed * m_Up;

	ComputeInfo();
}
/*--------------end of class cCamera implementation----------------*/

/********************************************************************
*	class viewport	implementation									*
*	By Shady el Mously												*
********************************************************************/

//	This will set the viewport
void sViewPort::Set(int startx,int starty,int width,int height)
{
	dwX=startx;
	dwY=starty;
	dwWidth=width;
	dwHeight=height;
	dvMinZ=0.0f;
	dvMaxZ=1.0f;		
}
/*------------end of class sViewport implementation----------------*/
			


/********************************************************************
*	class cBitmap	implementation									*
*	By Shady El Mously												*
********************************************************************/

//	initialize local data items
void cBitmap::Init(void)
{
	m_pImageData	=	NULL	;
	m_Width			=	0		;	
	m_Height		=	0		;
	m_RGBTYPE		=	GL_RGB	;
}

// LoadTexture()
// desc: loads a texture given the filename
void cBitmap::Load(char *fileName)
{
	char *extStr;

	// get extension from filename
	extStr = strchr(&fileName[1], '.');
	extStr++;

	// set the texture type based on extension of filename
	if ((strcmp(extStr, "BMP") == 0) || (strcmp(extStr, "bmp") == 0))
		LoadBMPFile(fileName);
	else if ((strcmp(extStr, "PCX") == 0) || (strcmp(extStr, "pcx") == 0) )
		LoadPCXFile(fileName);
	
	else if ((strcmp(extStr, "TGA") == 0) || (strcmp(extStr, "tga") == 0) )
		LoadTGAFile(fileName);
		//texType = TGA;
	
}




// LoadPCXFile()
// desc: loads a PCX image file as a texture
void cBitmap::LoadPCXFile(char *filename)
{
     PCXHEADER texInfo;            // header of texture
     unsigned char *unscaledData;// used to calculate pcx
     int i;                             // index counter
     int j;                             // index counter
     int width;                         // width of texture
     int height;                        // height of texture

     // load the PCX file into the texture struct
     m_pImageData = LoadPCXData(filename, &texInfo);

     // store the texture information
     m_Palette = texInfo.palette;
     m_Width			= texInfo.xMax - texInfo.xMin + 1;
     m_Height			= texInfo.yMax - texInfo.yMin + 1;
     textureType	= PCX;

     // allocate memory for the unscaled data
     unscaledData = (unsigned char*)malloc(m_Width*m_Height*4);

     // store the unscaled data via the palette
     for (j = 0; j < m_Height; j++) 
     {
          for (i = 0; i < m_Width; i++) 
          {
               unscaledData[4*(j*m_Width+i)+0] = (unsigned char)m_Palette[3*m_pImageData[j*m_Width+i]+0];
               unscaledData[4*(j*m_Width+i)+1] = (unsigned char)m_Palette[3*m_pImageData[j*m_Width+i]+1];
               unscaledData[4*(j*m_Width+i)+2] = (unsigned char)m_Palette[3*m_pImageData[j*m_Width+i]+2];
               unscaledData[4*(j*m_Width+i)+3] = (unsigned char)255;
          }
     }

     // find width and height's nearest greater power of 2
     width = m_Width;
     height = m_Height;

     // find width's
     i = 0;
     while (width)
     {
          width /= 2;
          i++;
     }
     m_ScaleHeight = (long)pow(2, i-1);

     // find height's
     i = 0;
     while (height)
     {
          height /= 2;
          i++;
     }
     m_ScaleWidth = (long)pow(2, i-1);

     // clear the texture data
     if (m_pImageData != NULL)
     {
          free(m_pImageData);
          m_pImageData = NULL;
     }

     // reallocate memory for the texture data
     m_pImageData = (unsigned char*)malloc(m_ScaleWidth*m_ScaleHeight*4);
     
     // use the GL utility library to scale the texture to the unscaled dimensions
     gluScaleImage (GL_RGBA, m_Width, m_Height, GL_UNSIGNED_BYTE, unscaledData, m_ScaleWidth, m_ScaleHeight, GL_UNSIGNED_BYTE, m_pImageData);

	 //	the type is RGBA
	 m_RGBTYPE		=	GL_RGBA	;
}



// LoadPCXData()
// desc: loads a PCX file into memory
unsigned char *cBitmap::LoadPCXData(char *filename, PCXHEADER *pcxHeader)
{
     int idx = 0;                  // counter index
     int c;                             // used to retrieve a char from the file
     int i;                             // counter index
     int numRepeat;      
     FILE *filePtr;                // file handle
     int width;                         // pcx width
     int height;                        // pcx height
     unsigned char *pixelData;     // pcx image data
     unsigned char *paletteData;   // pcx palette data

	 
     // open PCX file
     filePtr = fopen(filename, "rb");
     if (filePtr == NULL)
          return NULL;

     // retrieve first character; should be equal to 10
     c = getc(filePtr);
     if (c != 10)
     {
          fclose(filePtr);
          return NULL;
     }

     // retrieve next character; should be equal to 5
     c = getc(filePtr);
     if (c != 5)
     {
          fclose(filePtr);
          return NULL;
     }

     // reposition file pointer to beginning of file
     rewind(filePtr);

     // read 4 characters of data to skip
     fgetc(filePtr);
     fgetc(filePtr);
     fgetc(filePtr);
     fgetc(filePtr);

     // retrieve leftmost x value of PCX
     pcxHeader->xMin = fgetc(filePtr);       // loword
     pcxHeader->xMin |= fgetc(filePtr) << 8; // hiword

     // retrieve bottom-most y value of PCX
     pcxHeader->yMin = fgetc(filePtr);       // loword
     pcxHeader->yMin |= fgetc(filePtr) << 8; // hiword

     // retrieve rightmost x value of PCX
     pcxHeader->xMax = fgetc(filePtr);       // loword
     pcxHeader->xMax |= fgetc(filePtr) << 8; // hiword

     // retrieve topmost y value of PCX
     pcxHeader->yMax = fgetc(filePtr);       // loword
     pcxHeader->yMax |= fgetc(filePtr) << 8; // hiword

     // calculate the width and height of the PCX
     width = pcxHeader->xMax - pcxHeader->xMin + 1;
     height = pcxHeader->yMax - pcxHeader->yMin + 1;

     // allocate memory for PCX image data
     pixelData = (unsigned char*)malloc(width*height);

     // set file pointer to 128th byte of file, where the PCX image data starts
     fseek(filePtr, 128, SEEK_SET);
     
     // decode the pixel data and store
     while (idx < (width*height))
     {
          c = getc(filePtr);
          if (c > 0xbf)
          {
               numRepeat = 0x3f & c;
               c = getc(filePtr);

               for (i = 0; i < numRepeat; i++)
               {
                    pixelData[idx++] = c;
               }
          }
          else
               pixelData[idx++] = c;

          fflush(stdout);
     }

     // allocate memory for the PCX image palette
     paletteData = (unsigned char*)malloc(768);

     // palette is the last 769 bytes of the PCX file
     fseek(filePtr, -769, SEEK_END);

     // verify palette; first character should be 12
     c = getc(filePtr);
     if (c != 12)
     {
          fclose(filePtr);
          return NULL;
     }

     // read and store all of palette
     for (i = 0; i < 768; i++)
     {
          c = getc(filePtr);
          paletteData[i] = c;
     }

     // close file and store palette in header
     fclose(filePtr);
     pcxHeader->palette = paletteData;

     // return the pixel image data
     return pixelData;
}


//	Load a bitmap image form a 24 bit RGBbitmap
void cBitmap::LoadBMPFile(char* filename)
{
	FILE				*fptr;					//	File Pointer
	BITMAPINFOHEADER	bitmapInfoHeader;
	BITMAPFILEHEADER	bitmapFileHeader;		//	Bitmap File Header


	//	delete image if anything was previously loaded into it
	SAFE_FREE(m_pImageData);


	//	open the bitmap file as binary
	fptr	=	fopen(filename,"rb");
	if(!fptr)
		return ;

	//	read the bitmap file header
	fread(&bitmapFileHeader,sizeof(BITMAPFILEHEADER),1,fptr);

	//	m,ake sure this is a bitmap by checking for the universal bitmap id
	if(bitmapFileHeader.bfType!=0x4D42)
	{
		fclose(fptr);
		return ;
	}
	
	//	read the bitmap information header
	fread(&bitmapInfoHeader,sizeof(BITMAPINFOHEADER),1,fptr);

	//	move file pointer to beginning of bitmap data
	fseek(fptr,bitmapFileHeader.bfOffBits,SEEK_SET);

	//	allocate enough memory for the bitmap image data
	m_pImageData=new unsigned char[bitmapInfoHeader.biSizeImage];

	//	verify memory allocation
	if(!m_pImageData)
	{
		fclose(fptr);
		return ;
	}

	//	read in the bitmap image data..
	fread(m_pImageData,1,bitmapInfoHeader.biSizeImage,fptr);

	//	make sure the data was read
	if(!m_pImageData)
	{
		fclose(fptr);
		return ;
	}

	//	swap GBR to RGB using some assembly
	SwapColors(m_pImageData,bitmapInfoHeader.biWidth*bitmapInfoHeader.biHeight);
	fclose(fptr);

	//	save width an hright of the bitmap
	m_Width			=	bitmapInfoHeader.biWidth;
	m_Height		=	bitmapInfoHeader.biHeight;


	 //	the type is RGBA
	 m_RGBTYPE		=	GL_RGB;
}


//	Load an image from an uncompressed TGA file... still under test
int	cBitmap::LoadTGAFile(char* filename)
{
	FILE*	fptr;					//	the file pointer
	unsigned char ucharBad;			//	garbage unsigned char data
	short int sintBad;				//	garbage short int data
	long	imageSize;				//	size of TGA image
	int		colorMode;				//	4 for RGBA or 3 for RGB

	unsigned char	imageTypeCode;	//	type of tga file

	//	delete image if anything was previously loaded into it
	SAFE_FREE(m_pImageData);


	//	open the TGA file
	fptr	=	fopen(filename,"rb");
	if(!fptr)
		return 0;

	//	read the first two bytes we dont need
	fread(&ucharBad,sizeof(unsigned char),1,fptr);
	fread(&ucharBad,sizeof(unsigned char),1,fptr);

	//	read in image type
	fread(&imageTypeCode,sizeof(unsigned char),1,fptr);

	//	for our purposes	,	the image type should be 2 or 3
	if((imageTypeCode!=2)&&(imageTypeCode!=3))
	{
		fclose(fptr);
		return 0;
	}

	//	read 13 bytes of data we dont need
	fread(&sintBad,sizeof(short int),1,fptr);
	fread(&sintBad,sizeof(short int),1,fptr);
	fread(&ucharBad,sizeof(unsigned char),1,fptr);
	fread(&sintBad,sizeof(short int),1,fptr);
	fread(&sintBad,sizeof(short int),1,fptr);

	//	read image dimensions
	fread(&m_Width,sizeof(short int),1,fptr);
	fread(&m_Height,sizeof(short int),1,fptr);


	//	read image bit depth
	fread(&m_BitCount,sizeof(unsigned char),1,fptr);

	//	read one byte of data we dont need
	fread(&ucharBad,sizeof(unsigned char),1,fptr);

	//	color mode 3-> BGR , 4-> BGRA
	colorMode	=	m_BitCount	/	8;
	imageSize	=	m_Width	*	m_Height	* colorMode;

	//	allocate memory for image data
	m_pImageData	=	new unsigned char[imageSize];

	//	read in image data
	fread(m_pImageData,sizeof(unsigned char),imageSize,fptr);

	//	swap GBR to RGB using some assembly
	SwapColors(m_pImageData,m_Width*m_Height);


	fclose(fptr);
	return 1;
}

//	assembly routine to flip from bgr ro rgb
void cBitmap::SwapColors(void* buffer,int size)						// Flips The Red And Blue Bytes (256x256)
{
	void* b = buffer;						// Pointer To The Buffer
	int s=size;
	__asm								// Assembler Code To Follow
	{
		mov ecx, s					// Set Up A Counter (Dimensions Of Memory Block)
		mov ebx, b						// Points ebx To Our Data (b)
		label:							// Label Used For Looping
			mov al,[ebx+0]					// Loads Value At ebx Into al
			mov ah,[ebx+2]					// Loads Value At ebx+2 Into ah
			mov [ebx+2],al					// Stores Value In al At ebx+2
			mov [ebx+0],ah					// Stores Value In ah At ebx
			
			add ebx,3					// Moves Through The Data By 3 Bytes
			dec ecx						// Decreases Our Loop Counter
			jnz label					// If Not Zero Jump Back To Label
	}
}
/*---------------end of class cBitMap implementation---------------*/

/********************************************************************
*	class cTexture implementation									*
*	by shady el mously												*
********************************************************************/
//	constructs a skin from a bitmap file..
cTexture::cTexture(char* filename) : cBitmap(filename)
{
	//	save texturing state
	glPushAttrib(GL_TEXTURE_BIT);
	//	first generate the texture name
	glGenTextures(1,&m_TextureName);
	//	then create the texture object
	glBindTexture(GL_TEXTURE_2D,m_TextureName);
	//	set filtering..
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);	// Linear Filtering
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_LINEAR);	// Linear Filtering
	//	Load Image Data
	gluBuild2DMipmaps(GL_TEXTURE_2D,m_RGBTYPE,m_Width,m_Height,m_RGBTYPE,GL_UNSIGNED_BYTE,m_pImageData);
	//	restore texturing state
	glPopAttrib();

	//	delete the bitmap image data
	//SAFE_DELETE(m_pImageData);
}


//	speical functions to render to a texture

//	call this when u start rendering on a texture
void cTexture::StartDrawing()
{
	//	set the view port..
	glViewport(0,0,m_Width,m_Height);

	//	setting the projection
	glMatrixMode(GL_PROJECTION);

	//	first load identity..
	glLoadIdentity();

	//	set a squared prespective projection
	gluPerspective(45.0f,(float)m_Width/(float)m_Height,0.1f,160.0f);
	
	//	set view matrix to identity
	glMatrixMode(GL_MODELVIEW);

	//	clear
	glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
}

//	call this when finished rendering to the texture
void cTexture::EndDrawing()
{
	//	read the pixels into the texture :)
	glReadPixels(0,0,m_Width,m_Height,GL_RGB,GL_UNSIGNED_BYTE,m_pImageData);

	//	save texturing state
	glPushAttrib(GL_TEXTURE_BIT);

	//	then create the texture object
	glBindTexture(GL_TEXTURE_2D,m_TextureName);

	//	Load Image Data
	gluBuild2DMipmaps(GL_TEXTURE_2D,m_RGBTYPE,m_Width,m_Height,m_RGBTYPE,GL_UNSIGNED_BYTE,m_pImageData);
	//	restore texturing state
	glPopAttrib();

	//	setting the projection
	glMatrixMode(GL_PROJECTION);

	//	first load identity..
	glLoadIdentity();

	//	restore prespective..
	gluPerspective(45.0f,(GLfloat)Application()->m_Width/(GLfloat)Application()->m_Height,0.1f,160.0f);
	
	//	set view matrix to identity
	glMatrixMode(GL_MODELVIEW);

	//	restore viewport
	glViewport(0,0,Application()->m_Width,Application()->m_Height);

}
/*---------------end of class cTexture implementation--------------*/

/********************************************************************
*	class cMultiTexture implementation								*
*	by shady el mously												*
********************************************************************/



//	constructor will create 2 textures
cMultiTexture::cMultiTexture(char* Tex1name,char* Tex2name,int GenMode):m_Texture1(Tex1name),m_Texture2(Tex2name)
{
	//	if first object.. zabbat el denia..e3mel el functions we 3eesh
	if(!Graphics()->m_bCanMultiTexture)
		Application()->ReportError("No Support for MultiTextures");

	//	Store the Generation mode
	m_GenMode	=	GenMode;
}

//	activation of a multitexture...
void	cMultiTexture::Activate()
{
	glEnable(GL_TEXTURE_2D);

	glActiveTexture(GL_TEXTURE0_ARB);						//	set unit to zero
	glPushAttrib(GL_TEXTURE_BIT);								//	save texture state for unit zero
	glBindTexture(GL_TEXTURE_2D,m_Texture1.m_TextureName);		//	bind the first texture
	glTexEnvi(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_REPLACE);	//	this make first texure has no light effects

	glActiveTexture(GL_TEXTURE1_ARB);						//	set unit to one
	glPushAttrib(GL_TEXTURE_BIT);								//	save texture state for unit one
	glEnable(GL_TEXTURE_2D);									//	enable texturing here on unit 1
	glBindTexture(GL_TEXTURE_2D,m_Texture2.m_TextureName);		//	bind 2nd texture
	glTexEnvi(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_MODULATE);	//	set its parameters


	glTexGenf(GL_S,GL_TEXTURE_GEN_MODE,m_GenMode);
	glTexGenf(GL_T,GL_TEXTURE_GEN_MODE,m_GenMode);
	glEnable(GL_TEXTURE_GEN_S);
	glEnable(GL_TEXTURE_GEN_T);

	glActiveTexture(GL_TEXTURE0_ARB);
}

//	call when done using the texture
void cMultiTexture::Deactivate()
{
	glActiveTexture(GL_TEXTURE1_ARB);						//	set current unit to one
	glDisable(GL_TEXTURE_GEN_S);
	glDisable(GL_TEXTURE_GEN_T);
	glDisable(GL_TEXTURE_2D);
	glPopAttrib();												//	restore unit one state
	glActiveTexture(GL_TEXTURE0_ARB);						//	set current unit to zero
	glPopAttrib();												//	restore unit zero state
	glDisable(GL_TEXTURE_2D);
}
/*-----------end of class cMultiTexture implementation-------------*/

/********************************************************************
*	class cEnvTexture implementation								*
*	By Shady el mously												*
********************************************************************/
void cEnvTexture::Activate()
{
	cTexture::Activate();
	glTexGenf(GL_S,GL_TEXTURE_GEN_MODE,GL_SPHERE_MAP);
	glTexGenf(GL_T,GL_TEXTURE_GEN_MODE,GL_SPHERE_MAP);
	glEnable(GL_TEXTURE_GEN_S);
	glEnable(GL_TEXTURE_GEN_T);
}

void cEnvTexture::Deactivate()
{
	glDisable(GL_TEXTURE_GEN_S);
	glDisable(GL_TEXTURE_GEN_T);
	glDisable(GL_TEXTURE_2D);
}
/*------------------end of class cEnvTexture------------------------*/
/********************************************************************
*	class cEnvPassTexture implementation							*
*	By Shady el mously												*
********************************************************************/
void cEnvPassTexture::Activate()
{
	cTexture::Activate();
	glEnable(GL_BLEND);
	glBlendFunc(GL_ONE,GL_ONE);
	glTexGenf(GL_S,GL_TEXTURE_GEN_MODE,GL_SPHERE_MAP);
	glTexGenf(GL_T,GL_TEXTURE_GEN_MODE,GL_SPHERE_MAP);
	glEnable(GL_TEXTURE_GEN_S);
	glEnable(GL_TEXTURE_GEN_T);
}

void cEnvPassTexture::Deactivate()
{
	glDisable(GL_TEXTURE_GEN_S);
	glDisable(GL_TEXTURE_GEN_T);
	glDisable(GL_TEXTURE_2D);
	glDisable(GL_BLEND);
}
/*------------------end of class cEnvTexture------------------------*/
/********************************************************************
*	class cLinearTexture implementation								*
*	By Shady el mously												*
********************************************************************/
void cLinearTexture::Activate()
{
	cTexture::Activate();
	glTexGenf(GL_S,GL_TEXTURE_GEN_MODE,GL_OBJECT_LINEAR);
	glTexGenf(GL_T,GL_TEXTURE_GEN_MODE,GL_OBJECT_LINEAR);
	glEnable(GL_TEXTURE_GEN_S);
	glEnable(GL_TEXTURE_GEN_T);
	
}

void cLinearTexture::Deactivate()
{
	glDisable(GL_TEXTURE_GEN_S);
	glDisable(GL_TEXTURE_GEN_T);
	glDisable(GL_TEXTURE_2D);
}

/*------------------end of class cLinearTexture---------------------*/
/****************************************************************
*	class cFlatMirror implementation							*
****************************************************************/
//	call this and before drawing the reflected scene
void cMirrorFloor::StartDrawingReflecion(void)
{
	//	disable depth test
	glDisable(GL_DEPTH_TEST);

	//	disable modification of all color components
	glColorMask(GL_FALSE,GL_FALSE,GL_FALSE,GL_FALSE);

	//	enable stencil test
	glEnable(GL_STENCIL_TEST);
	
	//	set up the stencil buffer for a function reference value
	glStencilOp(GL_REPLACE,GL_REPLACE,GL_REPLACE);
	glStencilFunc(GL_ALWAYS,1,1);

	//	draw the floor on the stencil buffer
	Draw();

	//	enable color modification
	glColorMask(GL_TRUE,GL_TRUE,GL_TRUE,GL_TRUE);

	//	re enable z test
	glEnable(GL_DEPTH_TEST);

	//	make it so we can only render when stencil buffer is equal 1
	glStencilFunc(GL_EQUAL,1,1);
	glStencilOp(GL_KEEP,GL_KEEP,GL_KEEP);

	//	draw reflection
	glPushMatrix();
				
		glCullFace(GL_FRONT);
		glScalef(1,-1,1);
}

//	call this when done drawing reflection, it will draw the mirror itself
void cMirrorFloor::EndDrawingReflection(void)
{
	glCullFace(GL_BACK);
	glPopMatrix();
	glDisable(GL_STENCIL_TEST);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
	glColor4f(1,1,1,m_Transparency);
	glDisable(GL_LIGHTING);
	m_MirrorTexture.Activate();
	Draw();
	m_MirrorTexture.Deactivate();
	glEnable(GL_LIGHTING);
	glDisable(GL_BLEND);
}


//	this will draw the quad mirror itself
void cMirrorFloor::Draw()
{
	glBegin(GL_QUADS);
		glTexCoord2f(0.0,0.0); glVertex3f(-m_Width,0,m_Height);
		glTexCoord2f(0.0,4.0); glVertex3f(m_Width,0,m_Height);
		glTexCoord2f(4.0,4.0); glVertex3f(m_Width,0,-m_Height);
		glTexCoord2f(4.0,0.0); glVertex3f(-m_Width,0,-m_Height);
	glEnd();
}
/*------------end of class cFlatMirror implementation-------------*/

/********************************************************************
*	class cFog		implementation									*
*	By Shady el Mously												*
********************************************************************/


// set the fog parameters and type..
void cFog::Set(const color3& Color,float Start,float End,int Type,float Density)
{
	m_Color		=	Color;
	m_Type		=	Type;
	m_Start		=	Start;
	m_End		=	End;
	m_Density	=	Density;

	glFogf(GL_FOG_MODE,Type);
	glFogfv(GL_FOG_COLOR,Color.c);
	glFogf(GL_FOG_END,End);
	glFogf(GL_FOG_START,Start);
	glFogf(GL_FOG_DENSITY,Density);
}

/*-----------------------------------------------------------------*/
//		end of class fog implementation






