#include "meshes.h"

/********************************************************************
*	meshes.cpp														*
*	implementation of the mesh structs and classes					*
********************************************************************/

#include "opengl.h"		// <<--- needed for the 3d text

/********************************************************************
*	3D meshes classes implementation								*
*	By Shady El Mously												*
********************************************************************/


/********************************************************************
*	struct mesh methods..											*
********************************************************************/

//	constructor does nothing
sMesh::sMesh()
{
	m_Texture	=	0;	
}		


// this will get the bounding sphere radius
float sMesh::GenRadius()
{
	float best=0.0f;
	int size=m_Verts.size();

	for(int i=0;i<size;i++)
	{
		float curr=m_Verts[i].mag();
		if(curr>best)		best=curr;
	}
	return best;
}

//	this will twist the mesh around a certain vector..
void sMesh::Twist(float Angle,float axeX,float axeY,float axeZ)
{
	matrix4 TwistMatrix;

	point3d TwistAxis(axeX,axeY,axeZ);

	for(int i=0;i<m_Verts.size();i++)
	{
		TwistMatrix.isaxisrotation(TwistAxis,Angle*(m_Verts[i]*TwistAxis));
		m_Verts[i]=m_Verts[i]*TwistMatrix;
	}
}

//	scaling the normals is essential when scaling object matrices..
void sMesh::ScaleNormals(float ratio)
{
	for(int i=0;i<m_Verts.size();i++)
	{
		m_Norms[i]*=ratio;
	}
}
//	scaling the normals is essential when scaling object matrices, scaling in 3 dimensions
void sMesh::ScaleNormals(float Sx,float Sy,float Sz)
{
	for(int i=0;i<m_Verts.size();i++)
	{
		m_Norms[i].x*=Sx;
		m_Norms[i].y*=Sy;
		m_Norms[i].z*=Sz;
	}
}

//	this will shift the mesh in the local object space
void	sMesh::Translate(float x,float y,float z)
{
	for(int i=0;i<m_Verts.size();i++) 
	{
		m_Verts[i].x+=x;
		m_Verts[i].y+=y;
		m_Verts[i].z+=z;
	}
}

/*-----------------end of struct mesh implementation----------------*/


/********************************************************************
*	struct stridedmesh methods.										*
********************************************************************/

//	constructor..check if we support locking arrays and sets the essentail rendering function
sStridedMesh::sStridedMesh()
{

}

//	draw the srided mesh using vertex arrays, should be very fast
void sStridedMesh::Draw()
{
	if(Graphics()->m_bCanLockArrays)
	{
		glVertexPointer(3,GL_FLOAT,0,m_Verts.begin());				//	set vertex pointer to start of vertex array
		glNormalPointer(GL_FLOAT,0,m_Norms.begin());					//	set normals pointer to start of normals array
		glTexCoordPointer(2,GL_FLOAT,0,m_TexCoords.begin());			//	set texture coordinates pointer to start of texcoords array
		glLockArrays(0,m_Verts.size());								//	lock the arrays
		glDrawArrays(GL_TRIANGLE_STRIP,0,m_Verts.size());			//	Draw all the arrays as a triangle strip
		glUnlockArrays();
	}
	else
	{
		glVertexPointer(3,GL_FLOAT,0,m_Verts.begin());				//	set vertex pointer to start of vertex array
		glNormalPointer(GL_FLOAT,0,m_Norms.begin());					//	set normals pointer to start of normals array
		glTexCoordPointer(2,GL_FLOAT,0,m_TexCoords.begin());			//	set texture coordinates pointer to start of texcoords array
		glDrawArrays(GL_TRIANGLE_STRIP,0,m_Verts.size());			//	Draw all the arrays as a triangle strip
	}
}

//	this will create a sphere
void sStridedMesh::CreateSphere(float radius,int numSphereRings,int numSphereSegments)
{
	point3d		NewPoint	;
	point3d		NewNorm		;
	texcoord	NewTexCoord	;

	//	clear mesh data ..
	m_Verts.clear();
	m_Norms.clear();
	m_TexCoords.clear();

    // Establish constants used in sphere generation
    float fDeltaRingAngle = ( PI / numSphereRings );
    float fDeltaSegAngle  = ( 2.0f * PI / numSphereSegments );

    // Generate the group of rings for the sphere
    for( int ring = 0; ring < numSphereRings; ring++ )
    {    
	    float r0 = (float)sin( (ring+0) * fDeltaRingAngle );
        float r1 = (float)sin( (ring+1) * fDeltaRingAngle );
	    float y0 = (float)cos( (ring+0) * fDeltaRingAngle );
        float y1 = (float)cos( (ring+1) * fDeltaRingAngle );

		// Generate the group of segments for the current ring
        for( int seg = 0; seg < (numSphereSegments+1); seg++ )
	    {
		    float x0 =  r0 * (float)sin( seg * fDeltaSegAngle );
			float z0 =  r0 * (float)cos( seg * fDeltaSegAngle );
	        float x1 =  r1 * (float)sin( seg * fDeltaSegAngle );
            float z1 =  r1 * (float)cos( seg * fDeltaSegAngle );


			NewPoint	= radius * point3d(x0,y0,z0);
			NewNorm		= NewPoint;
			NewNorm.normalize();
			NewTexCoord.s =  ((float)seg)/numSphereSegments;
			NewTexCoord.t =  1-((ring+0)/(float)numSphereRings);

			m_Verts.push_back(NewPoint);
			m_Norms.push_back(NewNorm);
			m_TexCoords.push_back(NewTexCoord);
			

			NewPoint	= radius * point3d(x1,y1,z1);
			NewNorm		= NewPoint;
			NewNorm.normalize();

			NewTexCoord.s =  ((float)seg)/numSphereSegments;
			NewTexCoord.t =  1-((ring+1)/(float)numSphereRings);

			m_Verts.push_back(NewPoint);
			m_Norms.push_back(NewNorm);
			m_TexCoords.push_back(NewTexCoord);
		}
	}
}


/*----------end of struct stridedmesh implementation----------------*/

/********************************************************************
*	indexedmesh methods..											*
********************************************************************/

//	default constructor 
sIndexedMesh::sIndexedMesh()
{
}	

//	this will construct amesh from an o3d file..
sIndexedMesh::sIndexedMesh(const char * filename)
{

	FILE * file;		//	file pointer to mesh file (o3d)
	if(filename[strlen(filename)-1]=='t')	//	if terminating letter is T
	{
		file=fopen(filename,"r+t");			//	open as text
		if(!file)
			Application()->ReportError("%s Not Found!",filename);

		ReadMesh(file,false);
		fclose(file);
	}
	else									//	anything else
	{
		file=fopen(filename,"rb");			//	open as binary
		if(!file)
			Application()->ReportError("%s Not Found!",filename);

		ReadMesh(file,true);
		fclose(file);
	}
}

//	draw the mesh as indexed..using the m_Tris array to index, using also vertex arrays
void sIndexedMesh::Draw()
{
	if(Graphics()->m_bCanLockArrays)
	{
		glVertexPointer(3,GL_FLOAT,0,m_Verts.begin());				//	set vertex pointer to start of vertex array
		glNormalPointer(GL_FLOAT,0,m_Norms.begin());					//	set normals pointer to start of normals array
		glTexCoordPointer(2,GL_FLOAT,0,m_TexCoords.begin());			//	set texture coordinates pointer to start of texcoords array
		glLockArrays(0,m_Verts.size());								//	lock the arrays
		glDrawElements(GL_TRIANGLES,m_Tris.size()*3,					//	draw elements using
		GL_UNSIGNED_SHORT,m_Tris.begin());							//	the indexes sent
		glUnlockArrays();													//	unlock arrays
	}
	else
	{
		glVertexPointer(3,GL_FLOAT,0,m_Verts.begin());				//	set vertex pointer to start of vertex array
		glNormalPointer(GL_FLOAT,0,m_Norms.begin());					//	set normals pointer to start of normals array
		glTexCoordPointer(2,GL_FLOAT,0,m_TexCoords.begin());			//	set texture coordinates pointer to start of texcoords array
		glDrawElements(GL_TRIANGLES,m_Tris.size()*3,					//	draw elements using
		GL_UNSIGNED_SHORT,m_Tris.begin());							//	the indexes sent
	}
}


//	calculate vector normals for indexed meshes
void sIndexedMesh::ComputeNormals(vector<point3d>& points,vector<point3d>& norms)
{
	//first zero out all normals
	for(int i=0;i<norms.size();i++)
		norms[i].assign(0,0,0);

	//then go through and add each triangle normal to each of its vertices
	for(i=0;i<m_Tris.size();i++)
	{
		// construct a plane 
		plane3d plane(points[m_Tris[i].v[0]],
					points[m_Tris[i].v[1]],
					points[m_Tris[i].v[2]]);

		norms[m_Tris[i].v[0]]+=plane.n;
		norms[m_Tris[i].v[1]]+=plane.n;
		norms[m_Tris[i].v[2]]+=plane.n;
	}

	
	//finaly normalize all of the normals
	for(i=0;i<norms.size();i++)
		norms[i].normalize();	
}


//	this will twist the mesh around a certain vector..
void sIndexedMesh::Twist(float angle,float axeX,float axeY,float axeZ)
{
	sMesh::Twist(angle,axeX,axeY,axeZ);
	ComputeNormals(m_Verts,m_Norms);
}

//	read a mesh from a prviously opened file
void sIndexedMesh::ReadMesh(FILE * file,bool binary)
{

	int i;
	// read the first line... contains 5 fields
	cString::ReadTokenLine(file);
	// now start processing arg_c and arg_v[]
	// first string is the name
	m_Name=cString::arg_v[0];
	// next is the number of fields in the vertex info
	int numvertexfields=atoi(cString::arg_v[1]);
	// next is the triangle offset
	int offset=atoi(cString::arg_v[2]);
	// next is number of vertices
	int numverts=atoi(cString::arg_v[3]);
	// next is the triangle
	int numtriangles=atoi(cString::arg_v[4]);

	//	check if there is a texture name..
	if(cString::arg_v.size()==6)
	{
		m_Texture	=	new cTexture(cString::arg_v[5]);
	}


	point3d		NewPoint	;		
	point3d		NewNorm		;
	texcoord	NewTexCoord	;

	//	clear mesh data ..
	m_Verts.clear();
	m_Norms.clear();
	m_TexCoords.clear();
	m_Tris.clear();

	// reseve space in the verts vector
	m_Verts.reserve(numverts);
	m_Norms.reserve(numverts);
	m_TexCoords.reserve(numverts);


	// reserve space in the triangles vectors
	m_Tris.reserve(numtriangles);
	
	//	temporary vertex and triangles to be filled
	stri tri;
	//	a single letter will determine if we will continue reading as text or as binary data
	
	
	// now loop the same number of vertices..
	// each time
	// i must read a line
	// and fill its data
	if(binary)
	{
		for( i=0;i<numverts;i++)
		{
			switch(numvertexfields)
			{
			case 3:
				//	first 3 values are always guaranteed
				fread(&NewPoint.x,sizeof(float),1,file);
				fread(&NewPoint.y,sizeof(float),1,file);
				fread(&NewPoint.z,sizeof(float),1,file);

				NewNorm.assign(0,0,0);
				NewTexCoord.s=0.0f;
				NewTexCoord.t=0.0f;
			
				break;

			case 5:

				//	first 3 values are always guaranteed
				fread(&NewPoint.x,sizeof(float),1,file);
				fread(&NewPoint.y,sizeof(float),1,file);
				fread(&NewPoint.z,sizeof(float),1,file);

				fread(&NewTexCoord.s,sizeof(float),1,file);
				fread(&NewTexCoord.t,sizeof(float),1,file);

				
				NewNorm.assign(0,0,0);
				break;

			case 6:

				//	first 3 values are always guarantueed
				fread(&NewPoint.x,sizeof(float),1,file);
				fread(&NewPoint.y,sizeof(float),1,file);
				fread(&NewPoint.z,sizeof(float),1,file);

				fread(&NewNorm.x,sizeof(float),1,file);
				fread(&NewNorm.y,sizeof(float),1,file);
				fread(&NewNorm.z,sizeof(float),1,file);

				NewTexCoord.s=0.0f;
				NewTexCoord.t=0.0f;
				break;

			case 8:
				//	first 3 values are always guarantueed
				fread(&NewPoint.x,sizeof(float),1,file);
				fread(&NewPoint.y,sizeof(float),1,file);
				fread(&NewPoint.z,sizeof(float),1,file);

				fread(&NewNorm.x,sizeof(float),1,file);
				fread(&NewNorm.y,sizeof(float),1,file);
				fread(&NewNorm.z,sizeof(float),1,file);

				fread(&NewTexCoord.s,sizeof(float),1,file);
				fread(&NewTexCoord.t,sizeof(float),1,file);
			
				break;
		
			default:break;
			}	//	end of switch

			//	invert s coordinate
			NewTexCoord.s=1.0-NewTexCoord.s;


			// add the curr vertex to the vector
			m_Verts.push_back(NewPoint);
			m_Norms.push_back(NewNorm);
			m_TexCoords.push_back(NewTexCoord);
		}

		
		

		//	now read triangle indexing info
		for( i=0;i<numtriangles;i++)
		{
			if(feof(file))	Application()->ReportError("file ended");

			fread(&tri.v[0],sizeof(tri.v[0]),1,file);
			fread(&tri.v[1],sizeof(tri.v[1]),1,file);
			fread(&tri.v[2],sizeof(tri.v[2]),1,file);
			m_Tris.push_back(tri);
		}
	}
	else//text reading of the mesh..
	{
		for( i=0;i<numverts;i++)
		{
			// read a line and split its contents
			cString::ReadTokenLine(file);
			// vertex data is guranteed
			NewPoint.x=atof(cString::arg_v[0]);
			NewPoint.y=atof(cString::arg_v[1]);
			NewPoint.z=atof(cString::arg_v[2]);
			// load normals data if number of fields is 6 or 8
			if(numvertexfields==6 ||numvertexfields==8)
			{
				NewNorm.x=atof(cString::arg_v[3]);
				NewNorm.y=atof(cString::arg_v[4]);
				NewNorm.z=atof(cString::arg_v[5]);
			}
			else // no normal data
			{
				NewNorm.assign(0,0,0);
			}
			// load texture data if it is 5 or 8
			if(numvertexfields==5)
			{
				NewTexCoord.s=atof(cString::arg_v[3]);
				NewTexCoord.t=atof(cString::arg_v[4]);
			}
			else
			if(numvertexfields==8)
			{
				NewTexCoord.s=atof(cString::arg_v[6]);
				NewTexCoord.t=atof(cString::arg_v[7]);
			}	
			else
			{
				NewTexCoord.s=0.0f;
				NewTexCoord.t=0.0f;
			}

			//	invert s coordinate
			NewTexCoord.s=1.0-NewTexCoord.s;

			// now before reading next line.. add the curr vertex to the vetor
			m_Verts.push_back(NewPoint);
			m_Norms.push_back(NewNorm);
			m_TexCoords.push_back(NewTexCoord);
		}//	end for loop

		// now for the triangles info
		for( i=0;i<numtriangles;i++)
		{
			
			cString::ReadTokenLine(file);
			// also read a line.. and split it into tokens
			// fill the temp tri
			tri.v[0]=atoi (cString::arg_v[0]) - offset;
			tri.v[1]=atoi (cString::arg_v[1]) - offset;
			tri.v[2]=atoi (cString::arg_v[2]) - offset;
			// add it to the model vector
			m_Tris.push_back(tri);
		}// finihsed reading the indeces information
	}//	end text reading
	// make normals if they werent provided
	if(numvertexfields==3 || numvertexfields==5)
		ComputeNormals(m_Verts,m_Norms);
	// end if
	
}//	end of read mesh

//	this will create a box...
//	with an indexed mesh
//	i can specify the number of segments in each direction
void sIndexedMesh::CreateBox(float Width,float Height,float Depth,int numsegx,int numsegy,int numsegz,int numfaces)
{
	int i,j;	//for looping

	//	tempo data used when filling arrays
	point3d		NewPoint	;
	point3d		NewNorm		;
	texcoord	NewTexCoord	;

	//	clear mesh data ..
	m_Verts.clear();
	m_Norms.clear();
	m_TexCoords.clear();
	m_Tris.clear();


	//	triangle index
	tri<unsigned short> triangle;

	//	this vector will be used inside the loops as a bounding condition
	point3d slice;

	slice.x=numsegx;	slice.y=numsegy;	slice.z=numsegz;

	//	step for each face
	point3d steps;

	steps.x=Width/slice.x;	steps.y=Height/slice.y;	steps.z=Depth/slice.z;

	//	those help to correctly index the triangles generated
	int triindex=0;

	//	index of the constant coordinate and the variable ones
	int ct;	int	dx,dy;
							   
	//	the constant value in each face of the box
	float cons;

	point3d dimentions(Width,Height,Depth);

	//	loop on each of the 6 faces of a box
	for(int faces=0;faces<numfaces;faces++)
	{
		//	face check :

		//	0,1 are front and back faces
		if(faces<=1)	{ct=2;dx=0;dy=1;}
		else
		//	2,3 are left and right faces
		if(faces<=3)	{ct=0;dx=1;dy=2;}
		else
		//	4,5 are top and bottom faces
		if(faces<=5)	{ct=1;dx=2;dy=0;}

		//	the constant may be positive or negative according to the face side
		if(faces%2==0)		cons=-dimentions.v[ct]/2;
		else				cons=+dimentions.v[ct]/2;

		//	now start making the face...
		//	each face is a number of quads..
		for (i=0;i<slice.v[dx];i++)
			for (j=0;j<slice.v[dy];j++)
			{
				//	now creating each quad in the side...

				//	0---> up left point
				NewPoint.v[dx]=(-dimentions.v[dx]/2)+(steps.v[dx]*i);
				NewPoint.v[dy]=(dimentions.v[dy]/2)-(steps.v[dy]*j);
				NewPoint.v[ct]=cons;
				NewTexCoord.s=(1.0/slice.v[dx])*i;
				NewTexCoord.t=(1.0/slice.v[dy])*j;

				m_Verts.push_back(NewPoint);
				m_Norms.push_back(NewNorm);
				m_TexCoords.push_back(NewTexCoord);

				

				//	1---> up right point
				NewPoint.v[dx]=(-dimentions.v[dx]/2)+(steps.v[dx]*(i+1));
				NewPoint.v[dy]=(dimentions.v[dy]/2)-(steps.v[dy]*j);
				NewPoint.v[ct]=cons;
				NewTexCoord.s=(1.0/slice.v[dx])*(i+1);	
				NewTexCoord.t=(1.0/slice.v[dy])*(j);
				
				m_Verts.push_back(NewPoint);
				m_Norms.push_back(NewNorm);
				m_TexCoords.push_back(NewTexCoord);

				

				//	2---> down right point
				NewPoint.v[dx]=(-dimentions.v[dx]/2)+(steps.v[dx]*(i+1));
				NewPoint.v[dy]=(dimentions.v[dy]/2)-(steps.v[dy]*(j+1));
				NewPoint.v[ct]=cons;
				NewTexCoord.s=(1.0/slice.v[dx])*(i+1);
				NewTexCoord.t=(1.0/slice.v[dy])*(j+1);

				m_Verts.push_back(NewPoint);
				m_Norms.push_back(NewNorm);
				m_TexCoords.push_back(NewTexCoord);

				

				//	3---> down left point
				NewPoint.v[dx]=(-dimentions.v[dx]/2)+(steps.v[dx]*i);
				NewPoint.v[dy]=(dimentions.v[dy]/2)-(steps.v[dy]*(j+1));
				NewPoint.v[ct]=cons;
				NewTexCoord.s=(1.0/slice.v[dx])*(i);
				NewTexCoord.t=(1.0/slice.v[dy])*(j+1);
				
				m_Verts.push_back(NewPoint);
				m_Norms.push_back(NewNorm);
				m_TexCoords.push_back(NewTexCoord);

				//	now index the 4 points just calculated
				//	and take into consideration the culling direction
				if(faces%2==0)
				{
					triangle.v[0]=0+triindex;triangle.v[1]=1+triindex;triangle.v[2]=2+triindex;	m_Tris.push_back(triangle);
					triangle.v[0]=2+triindex;triangle.v[1]=3+triindex;triangle.v[2]=0+triindex;	m_Tris.push_back(triangle);
				}
				else
				{
					triangle.v[0]=2+triindex;triangle.v[1]=1+triindex;triangle.v[2]=0+triindex;	m_Tris.push_back(triangle);
					triangle.v[0]=0+triindex;triangle.v[1]=3+triindex;triangle.v[2]=2+triindex;	m_Tris.push_back(triangle);
				}
				//	update the tri index by 4 so next turn the indexing is correct
				triindex+=4;
			}//	end of the two internal for loops
	}//	end of the faces loop

	//	compute vector normals and scale the textures...
	ComputeNormals(m_Verts,m_Norms);
}
/*--------end of struct indexedmesh implementation------------------*/
/****************************************************************
*	struct shiermesh implementation								*
*	By Shady el mously											*
****************************************************************/

//	always construct with an h3d file
sHierMesh::sHierMesh(char* filename)
{
	m_pNextMesh		=	m_pChildMesh	=	m_pParentMesh	=	0;
	m_bHasAlpha		=	false;	
	if(filename)Load(filename);
}

//	loads a hierarchical file in the tree mesh
void sHierMesh::Load(char* filename)
{
	FILE* infile;

	//	use filename to check is it is binary or text
	if(filename[strlen(filename)-1]=='t')
	{
		infile=fopen(filename,"r+t");
		if(!infile)
			Application()->ReportError("%s Not Found!",filename);
		LoadHierarchy(infile,false);
	}
	else
	{
		infile=fopen(filename,"r+b");
		if(!infile)
			Application()->ReportError("%s Not Found!",filename);
		LoadHierarchy(infile,true);
	}
	fclose(infile);
}


//	loads the single mesh data from a file...binary or text...
void sHierMesh::LoadSingleMeshData(FILE * fptr,bool binary)
{
	int i;

	//	first load the curent mesh data
	sIndexedMesh::ReadMesh(fptr,binary);

	//	then read the current matrix in..
	if(binary)
	{
		//	read the matrix as binary
		for(i=0;i<4;i++)
		{
			fread(&m_CurMatrix.m[i][0],sizeof(float),1,fptr);
			fread(&m_CurMatrix.m[i][1],sizeof(float),1,fptr);
			fread(&m_CurMatrix.m[i][2],sizeof(float),1,fptr);
			fread(&m_CurMatrix.m[i][3],sizeof(float),1,fptr);
		}

		//	check if there is material data
		cString::ReadTokenLine(fptr);
		//	if there is another token...enable alpha in this mesh
		if(cString::arg_v.size()>1)
			m_bHasAlpha		=	true;

		if(cString::arg_v[0]=="y")
		{
			fread(&m_Material.diffuse,sizeof(color4),1,fptr);
			fread(&m_Material.ambient,sizeof(color4),1,fptr);
			fread(&m_Material.specular,sizeof(color4),1,fptr);
			fread(&m_Material.emissive,sizeof(color4),1,fptr);
			fread(&m_Material.power,sizeof(float),1,fptr);
		}
	}
	else
	{
		//	read the data here as text
		for(i=0;i<4;i++)
		{
			cString::ReadTokenLine(fptr);
			m_CurMatrix.m[i][0]	=	atof(cString::arg_v[0]);
			m_CurMatrix.m[i][1]	=	atof(cString::arg_v[1]);
			m_CurMatrix.m[i][2]	=	atof(cString::arg_v[2]);
			m_CurMatrix.m[i][3]	=	atof(cString::arg_v[3]);
		}
		//	check if there is material info..
		cString::ReadTokenLine(fptr);

		//	if there is another token...enable alpha in this mesh
		if(cString::arg_v.size()>1)
			m_bHasAlpha		=	true;

		if(cString::arg_v[0]=="y")
		{
			//	if there are some vertices.. read material also..
			if(m_Verts.size()>1)
			{
				cString::ReadTokenLine(fptr);
				m_Material.diffuse.r=atof(cString::arg_v[0]);
				m_Material.diffuse.g=atof(cString::arg_v[1]);
				m_Material.diffuse.b=atof(cString::arg_v[2]);
				m_Material.diffuse.a=atof(cString::arg_v[3]);
	
				cString::ReadTokenLine(fptr);
				m_Material.ambient.r=atof(cString::arg_v[0]);
				m_Material.ambient.g=atof(cString::arg_v[1]);
				m_Material.ambient.b=atof(cString::arg_v[2]);
				m_Material.ambient.a=atof(cString::arg_v[3]);

				cString::ReadTokenLine(fptr);
				m_Material.specular.r=atof(cString::arg_v[0]);
				m_Material.specular.g=atof(cString::arg_v[1]);
				m_Material.specular.b=atof(cString::arg_v[2]);
				m_Material.specular.a=atof(cString::arg_v[3]);

				cString::ReadTokenLine(fptr);
				m_Material.emissive.r=atof(cString::arg_v[0]);
				m_Material.emissive.g=atof(cString::arg_v[1]);
				m_Material.emissive.b=atof(cString::arg_v[2]);
				m_Material.emissive.a=atof(cString::arg_v[3]);

				cString::ReadTokenLine(fptr);
				m_Material.power=atof(cString::arg_v[0]);

			}
		}
	}
}

//	recursive function that will read the hierarchy..
void sHierMesh::LoadHierarchy(FILE* fptr,bool binary)
{
	//	first load this data for this mesh itself
	LoadSingleMeshData(fptr,binary);

	//	if file is done.. go back..
	if(feof(fptr))return;

	//	check for further hierarchy data
	int z;

	cString::ReadTokenLine(fptr);
	z=atoi(cString::arg_v[0]);
	
	switch(z)
	{
	case 0:	//	nothing...
		return;
		break;

	case 1://	child only
		SAFE_DELETE(m_pChildMesh);
		m_pChildMesh	=	new sHierMesh();
		m_pParentMesh	=	this;
		break;

	case 2://	next only
		SAFE_DELETE(m_pNextMesh);
		m_pNextMesh		=	new sHierMesh();
		m_pParentMesh	=	this;
		break;

	case 3://	next and child
		SAFE_DELETE(m_pChildMesh);
		m_pChildMesh	=	new sHierMesh();
		m_pParentMesh	=	this;
		SAFE_DELETE(m_pNextMesh);
		m_pNextMesh		=	new sHierMesh();
		m_pParentMesh	=	this;
		break;

	default :
		Application()->ReportError("error in heirarchy file");
		break;
	}

	//	now recurse down the tree
	if(m_pChildMesh)
		m_pChildMesh->LoadHierarchy(fptr,binary);

	if(m_pNextMesh)
		m_pNextMesh->LoadHierarchy(fptr,binary);
}

//	the virtual drawing function..
void sHierMesh::Draw()
{
	//	if there any children? multiply local matrix and recurse
	if(m_pChildMesh)
	{
		glPushMatrix();
		glMultMatrixf(m_CurMatrix.d);
		m_pChildMesh->Draw();

		glPopMatrix();
	}
			
	//	any next meshes?..get using the current matrix and recurse
	if(m_pNextMesh)
		m_pNextMesh->Draw();
	
	glPushMatrix();
	glMultMatrixf(m_CurMatrix.d);

	m_Material.Activate();

	if(m_Texture)m_Texture->Activate();

	//	if there is alpha, actiavte alpha and disable z buffering
	if(m_bHasAlpha)
	{
		glDisable(GL_DEPTH_TEST);							//	disable zbuffer
		glEnable(GL_BLEND);									//	enable blending
		sIndexedMesh::Draw();								//	draw the mesh
		glEnable(GL_DEPTH_TEST);							//	disable zbuffer
		glDisable(GL_BLEND);								//	disable zbuffer
	}
	else
		sIndexedMesh::Draw();

	if(m_Texture)m_Texture->Deactivate();

	glPopMatrix();
}

//	this will scale all the object normals by recusring inside the tree..
void sHierMesh::ScaleTree(float sX,float sY,float sZ)
{
	sMesh::ScaleNormals(sX,sY,sZ);
	//	are there any children? use the new matrix and recurse
	if(m_pChildMesh)
		m_pChildMesh->ScaleTree(sX,sY,sZ);
			
	//	any next meshes?..get using the current matrix and recurse
	if(m_pNextMesh)
		m_pNextMesh->ScaleTree(sX,sY,sZ);
}

//	get address of a mesh witha certain name
sHierMesh	*	sHierMesh::GetMeshNamed(char* MeshName)
{
	//	first check here..
	if(strcmp(m_Name.c_str(),MeshName)==0)
		return this;

	if(m_pNextMesh)
	{
		return m_pNextMesh->GetMeshNamed(MeshName);
	}

	if(m_pChildMesh)
	{
		return m_pChildMesh->GetMeshNamed(MeshName);
	}
	return 0;
}
/*-----------end of hierarchical mesh implementation----------------*/


/********************************************************************
*	struct watermesh												*
********************************************************************/
sWaterMesh::sWaterMesh()
{
	m_Heights1		=	NULL		;
	m_Heights2		=	NULL		;
	m_DampRatios	=	NULL		;
}

sWaterMesh::sWaterMesh(float Width,float Height,int numColumns,int numRows)
{
	m_Width		=	numColumns;
	m_Height	=	numRows;


	int i,j;


	m_Dims.x=Width;
	m_Dims.y=0;
	m_Dims.z=Height;
	//	calculate the position of the starting point..
	//	always up left
	//	the next is in the plane z=0;
	m_Start.x=Width/2		;
	m_Start.y=0			;
	m_Start.z=Height/2	;

	m_Start.x=-m_Start.x;
	m_Start.z=-m_Start.z;


	//	allocate the 3 arrays used to make the water effect
	m_Heights1		=	new float*[m_Width];
	m_Heights2		=	new float*[m_Width];
	m_DampRatios	=	new float*[m_Width];


	for(i=0;i<numColumns;i++)
	{
		m_Heights1[i]	=	new float[m_Height];
		m_Heights2[i]	=	new float[m_Height];
		m_DampRatios[i]	=	new float[m_Height];
	}

	//	this is step square in the net
	
	m_Step.y=0;
	m_Step.x=Width/numColumns;
	m_Step.z=Height/numRows;

	//	now fill the two 2D arrays
	for(i=0;i<numColumns;i++)
		for(j=0;j<numRows;j++)
		{
			m_Heights1[i][j]=m_Heights2[i][j]=0.0f;
			m_DampRatios[i][j]=1.00f;
		}

	Randomize(0.001f);
}


//	destroy the data
sWaterMesh::~sWaterMesh()
{
	if(m_Heights1)
	{
		//	deallocate the two 2d arrays
		for(int i=0;i<m_Width;i++)
			delete [] m_Heights1[i];
		delete [] m_Heights1;
	}

	if(m_Heights2)
	{
		//	deallocate the two 2d arrays
		for(int i=0;i<m_Width;i++)
			delete [] m_Heights2[i];
		delete [] m_Heights2;
	}
	if(m_DampRatios)
	{
		//	deallocate the two 2d arrays
		for(int i=0;i<m_Width;i++)
			delete [] m_DampRatios[i];
		delete [] m_DampRatios;
	}
}


//	setup some randomz values for ther surface items..
//	negative or psitive.. send the maximim value of Z
void sWaterMesh::Randomize(float range)
{
	srand(GetTickCount());
	int n;

	//i have noew 500 pistive or 500 negative value..
	for(int i=1;i<m_Width-1;i++)
		for(int j=1;j<m_Height-1;j++)
		{
			n=(rand()%1000)-500;
			m_Heights1[i][j]=n/500.0f*range;
		}
}


//	draw the water mesh as indexed quadsusing the m_QuadINdexes array to index, using also vertex arrays
void sWaterMesh::Draw()
{
	Process(0.1f);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA,GL_ONE);
	glDisable(GL_LIGHTING);
//	glEnable(GL_TEXTURE_GEN_S);
//	glEnable(GL_TEXTURE_GEN_T);
//	glTexGenf(GL_S,GL_TEXTURE_GEN_MODE,GL_OBJECT_LINEAR);
//	glTexGenf(GL_T,GL_TEXTURE_GEN_MODE,GL_OBJECT_LINEAR);

	glRotatef(90,1,0,0);
    glBegin(GL_QUAD_STRIP);
	glNormal3f(0,1,0);
	int x,z;
    for(x = 0; x < m_Width-1; x++)
    {
		if(x%2)
		{
        for(z = 0; z < m_Height; z++)
			{
				glVertex3f(m_Start.x+((x+1)*m_Step.x),m_Start.z+(z*m_Step.z), m_Heights1[x+1][z]);
				glVertex3f(m_Start.x+(x*m_Step.x),m_Start.z+(z*m_Step.z), m_Heights1[x][z]);
			}
			glVertex3f(m_Start.x+((x+1)*m_Step.x),m_Start.z+(z*m_Step.z), m_Heights1[x+1][z]);
			glVertex3f(m_Start.x+(x*m_Step.x),m_Start.z+(z*m_Step.z), m_Heights1[x][z]);
		}
		else
		{
		    for(z = m_Height; z > 0; z--)
	        {
				glVertex3f(m_Start.x+(x*m_Step.x),m_Start.z+(z*m_Step.z), m_Heights1[x][z]);
	            glVertex3f(m_Start.x+((x+1)*m_Step.x),m_Start.z+(z*m_Step.z), m_Heights1[x+1][z]);
			}
			glVertex3f(m_Start.x+(x*m_Step.x),m_Start.z+(z*m_Step.z), m_Heights1[x][z]);
			glVertex3f(m_Start.x+((x+1)*m_Step.x),m_Start.z+(z*m_Step.z), m_Heights1[x+1][z]);
		}
    }
    glEnd();
//	glDisable(GL_TEXTURE_GEN_S);
//	glDisable(GL_TEXTURE_GEN_T);
	glEnable(GL_LIGHTING);
	glDisable(GL_BLEND);
}


//	this will process the water effect on a perframe basis
void sWaterMesh::Process(float c)
{
	const float dth=0.1f;
	

	const float A=(c*dth)*(c*dth);
	const float B=2-4*A;

	long i,j;

	for(i=1;i<m_Width-1;i++)
		for(j=1;j<m_Height-1;j++)
		{
			m_Heights1[i][j]
				=
				A*(m_Heights2[i-1][j]+m_Heights2[i+1][j]+m_Heights2[i][j-1]+m_Heights2[i][j+1])
				+
				B*m_Heights2[i][j]
				-
				m_Heights1[i][j];
		
			//	damp
			m_Heights1[i][j]*=m_DampRatios[i][j];
		}
		//	swap pointers
		swap(m_Heights1,m_Heights2);
}

//	this will set the damping ratio of the water..
void sWaterMesh::SetDamping(float value)
{
	for(int x=1;x<m_Width-1;x++)
		for(int y=1;y<m_Height-1;y++)
			m_DampRatios[x][y]=value;
}

// use this to scale the model..
void sWaterMesh::Scale(float amt)
{
	m_Dims	*=amt;
	m_Start	*=amt;
	m_Step	*=amt;
}



//	splash at position
void sWaterMesh::Splash(point3d& loc,float power)
{
	int i	=	-(loc.x	-	m_Dims.x)* m_Width /(2*m_Dims.x);
	int j	=	-(loc.z	-	m_Dims.z)* m_Height /(2*m_Dims.z);

	//	this will ensure the addressing is inside the water ..
	if(i>0&&j>0&&i<m_Width-1&&j<m_Height-1)
	{
		m_Heights1[i][j]=power;
		m_Heights2[i][j]=power;
	}
}


//	this will return a specified point..
float& sWaterMesh::HeightAt(point3d& loc)
{
	int i	=	-(loc.x	-	m_Dims.x)* m_Width /(2*m_Dims.x);
	int j	=	-(loc.z	-	m_Dims.z)* m_Height /(2*m_Dims.z);
	return m_Heights1[i][j];
}
/*--------------end of struct sWaterMesh implementation----------------*/

/************************************************************************
*	struct sTerrainMesh	implementation									*
*	By Shady El Mously													*
*	Generating a 3D map from a 2d grayscale bitmap						*
************************************************************************/

//	construct from a 24 bit grayscale bitmap file..
sTerrainMesh::sTerrainMesh(float Width,float Depth,char* TerrainFile,float HeightScale)
{
	int i,j;
	
	m_Dims.x=Width;
	m_Dims.y=Depth;
	m_Dims.z=0;

 
	int			MapSize;

	//	first make a bitmap from the filename sent
	cBitmap	Map(TerrainFile);


	//	calculate the point form which the loop will start
	point3d	startpoint=m_Dims/2;
	startpoint.z=0;
	startpoint.x=-startpoint.x;
	startpoint.y=-startpoint.y;

	point3d endpoint=-startpoint;


	MapSize	=	Map.m_Width;

	point3d step=m_Dims/MapSize;
	m_Step=step;

	//	allocate the heights..
	m_Heights1	=	new float*[MapSize];

	for(i=0;i<MapSize;i++)
		m_Heights1[i]=new float[MapSize];

	for(i=0;i<MapSize;i++)
		for(j=0;j<MapSize;j++)
		{
			m_Heights1[i][j]=(Map.m_pImageData[(i+j*Map.m_Width)*3]+
				Map.m_pImageData[1+(i+j*Map.m_Width)*3]+
				Map.m_pImageData[2+(i+j*Map.m_Width)*3])/3.0f*HeightScale;
		}


	int x,z;

    m_ListID=glGenLists(1);

    glNewList(m_ListID,GL_COMPILE);               

    
    glBegin(GL_QUAD_STRIP);
	glNormal3f(0,1,0);

    for(x = 0; x < MapSize-1; x++)
    {
		if(x%2)
		{
        for(z = 0; z < MapSize; z++)
			{
				glTexCoord2f((x+1)%2,z%2);
				glVertex3f(startpoint.x+((x+1)*step.x),m_Heights1[x+1][z],startpoint.y+(z*step.y));
				glTexCoord2f(x%2,z%2);
				glVertex3f(startpoint.x+(x*step.x),m_Heights1[x][z],startpoint.y+(z*step.y));
			}
			glTexCoord2f((x+1)%2,z%2);
			glVertex3f(startpoint.x+((x+1)*step.x),m_Heights1[x+1][z],startpoint.y+(z*step.y));
			glTexCoord2f(x%2,z%2);
			glVertex3f(startpoint.x+(x*step.x),m_Heights1[x][z],startpoint.y+(z*step.y));
		}
		else
		{
		    for(z = MapSize; z > 0; z--)
	        {
				glTexCoord2f(x%2,z%2);
				glVertex3f(startpoint.x+(x*step.x),m_Heights1[x][z],startpoint.y+(z*step.y));
				glTexCoord2f((x+1)%2,z%2);
	            glVertex3f(startpoint.x+((x+1)*step.x),m_Heights1[x+1][z],startpoint.y+(z*step.y));
			}
			glTexCoord2f(x%2,z%2);
			glVertex3f(startpoint.x+(x*step.x),m_Heights1[x][z],startpoint.y+(z*step.y));
			glTexCoord2f((x+1)%2,z%2);
			glVertex3f(startpoint.x+((x+1)*step.x),m_Heights1[x+1][z],startpoint.y+(z*step.y));
		}
    }

    glEnd();
    glEndList();    

}

//	draw the terrain
void sTerrainMesh::Draw()
{
    glCallList(m_ListID);
}


sTerrainMesh::~sTerrainMesh()
{
	if(m_ListID)
	glDeleteLists(m_ListID,1);
}


//	send an address of a water mesh to the terrain and it will adjust the dampings to zero over the hight
void sTerrainMesh::MakeSea(sWaterMesh* pWater,float SeaLevel)
{
	for(int i=0;i<pWater->m_Width;i++)
		for(int j=0;j<pWater->m_Width;j++)
		{
			if(m_Heights1[i][j]>SeaLevel)
				pWater->m_DampRatios[i][j]=0;
		}
}
/*--------------end of struct sTerrainMesh implementation--------------*/


/************************************************************************
*	struct sQuadricMesh	 implementation									*
*	encapsulates the OpenGL Quadric,									*
*	used as a base class for the disk,sphere and cylinder meshes		*
************************************************************************/

//	constrcutor always makes the quadric object
sQuadricMesh::sQuadricMesh()
{
	m_Quadric	=	gluNewQuadric();
	//	use smooth shading
	gluQuadricNormals(m_Quadric,GLU_SMOOTH);
	//	ouside
	gluQuadricOrientation(m_Quadric,GLU_OUTSIDE);
	//	use texture coordiantes
	gluQuadricTexture(m_Quadric,GL_TRUE);
}


/*---------------end of struct sQuadricMesh implementation-------------*/


//	this will draw the quadric as a disk
void sCylinderMesh::Draw()
{
	//	adjust everything so the cylinder is standing..
	glPushMatrix();
	glTranslatef(0,m_Height/2,0);
	glRotatef(90,1,0,0);
	gluCylinder(m_Quadric,m_BaseRadius,m_TopRadius,m_Height,20,20);
	glTranslatef(0,0,m_Height);
	gluDisk(m_Quadric,0,m_TopRadius,20,20);
	glTranslatef(0,0,-m_Height);
	glRotatef(180,1,0,0);
	gluDisk(m_Quadric,0,m_BaseRadius,20,20);
	glPopMatrix();
}

//	this will draw the quadric as a disk
void sDiskMesh::Draw()
{
	gluDisk(m_Quadric,m_InnerRadius,m_OuterRadius,30,20);
	glPushMatrix();
	glRotatef(180,1,0,0);
	gluDisk(m_Quadric,m_InnerRadius,m_OuterRadius,30,20);
	glPopMatrix();
}


//	draw the torus
void sTorusMesh::Draw()
{
	auxSolidTorus(m_Radius1,m_Radius2);
}
/************************************************************************
*	struct sTextMesh Implementation										*
************************************************************************/

//	this will draw the text mesh..normalizing normals is essential because it coulbe scales
void sTextMesh::Draw()
{

	if(!m_String)		return;			//	if no string.. go back

	length=0;					//	calculate here length of the string in floats
	
	for(idx=0;idx<strlen(m_String);idx++)//	calculate the length
		length+=GraphicsGL()->m_GMF[m_String[idx]].gmfCellIncX;

	glPushAttrib(GL_LIST_BIT);								//	save texturing state

	glPushMatrix();
	glTranslatef(-length/2.0f,-0.2f,GraphicsGL()->m_OutLineFontDepth/2.0f);//	translate to center the string
	glListBase(GraphicsGL()->m_OutLineFontBase);			//	set new list base
	glCallLists(strlen(m_String),GL_UNSIGNED_BYTE,m_String);		//	call lists that renders 3d text
	
	glPopMatrix();											//	restore previous matrix
	glPopAttrib();											//	restore display list bit

	glFrontFace(GL_CCW);									//	the culling is clock wise
}
/*-----------------end of struct sTextMesh implementation--------------*/


/************************************************************************
*	struct sStarsMesh implementation									*
************************************************************************/

#define MAX_RAND 		4096

//---------------------------------------------------------------------------
// Initialize stars
//---------------------------------------------------------------------------
void sStarsMesh::Init() {
	int step = cnt / 3;
	int i;

	glListID = glGenLists(1);

	glNewList(glListID, GL_COMPILE);

	glPushAttrib(GL_TEXTURE_BIT);
	glPushAttrib(GL_LIGHTING_BIT);
	glDisable(GL_LIGHTING);
	glDisable(GL_TEXTURE_2D);
	glDisable(GL_DEPTH_TEST);

	glColor3f(1.0, 1.0, 1.0);
	glPointSize(0.5);
	glBegin(GL_POINTS);
		for (i = 0; i < step; i++) {
			glColor3f(1.0, 1.0, 1.0);
			glVertex3f(
				radius * ((float)(rand() % MAX_RAND)/(MAX_RAND)) - radius / 2.0,
				radius * ((float)(rand() % MAX_RAND)/(MAX_RAND)) - radius / 2.0,
				radius * ((float)(rand() % MAX_RAND)/(MAX_RAND)) - radius / 2.0
			);
		}
	glEnd();
	glPointSize(1.0);
	glBegin(GL_POINTS);
		for (i = 0; i < step; i++) {
			glVertex3f(
				radius * ((float)(rand() % MAX_RAND)/(MAX_RAND)) - radius / 2.0,
				radius * ((float)(rand() % MAX_RAND)/(MAX_RAND)) - radius / 2.0,
				radius * ((float)(rand() % MAX_RAND)/(MAX_RAND)) - radius / 2.0
			);
		}
	glEnd();
	glPointSize(1.5);
	glBegin(GL_POINTS);
		for (i = 0; i < step; i++) {
			glVertex3f(
				radius * ((float)(rand() % MAX_RAND)/(MAX_RAND)) - radius / 2.0,
				radius * ((float)(rand() % MAX_RAND)/(MAX_RAND)) - radius / 2.0,
				radius * ((float)(rand() % MAX_RAND)/(MAX_RAND)) - radius / 2.0
			);
		}
	glEnd();
	glEnable(GL_DEPTH_TEST);
	glPopAttrib();
	glPopAttrib();
	glEndList();
}

/*------------------end of sStarsMesh implementation-------------------*/
