#ifndef _SCENES_H
#define _SCENES_H


#include "3dui.h"
#include "3dpaths.h"
#include "audio.h"
#include "special.h"
#include "helmet.h"		//	for camera controllers


/********************************************************************
*	struct sPathLink												*
*	reprsent a link between a path and a point3d struct				*
********************************************************************/
struct sPathLink
{
	point3d*	m_pPoint;
	cPath*		m_pPath;
	float		m_Scale;

	//	make them both nulls initially
	sPathLink()
	{
		m_pPoint	=	NULL	;
		m_pPath		=	NULL	;
		m_Scale		=	1.0f	;
	}
};

/*--------------end of struct sPathLink declaration----------------*/



/********************************************************************
*	struct sPointLink												*
*	has two pointer to point3d structs								*
*	takes data from source and insert it into destination			*
********************************************************************/
struct sPointLink
{
	point3d	*m_pSrcPoint	;	//	the source
	point3d	*m_pDestPoint	;	//	the destination
	float		m_Scale		;	//	scale of the link

	//	constructor set nulls
	sPointLink()
	{
		m_pSrcPoint	=	m_pDestPoint	=	NULL;
		m_Scale		=	1.0f;
	}
};
/*-----------------end of sPointLink declaration-------------------*/

/********************************************************************
*	class cSceneData												*
*	used to hold the data for a 3d scene							*
*	the scene must be described in a text file						*
*	this base class will be used to make simple static scenes		*
*	containing objects, lights,a camera								*
********************************************************************/
class cSceneData
{
protected :
	
	//	those should always be destroyed ( delete should be calles on them all
	vector<cRigidBody*>			m_ObjectsList		;	//	objects in the scene
	vector<sLight*>				m_LightsList		;	//	lights in the scene
	vector<cPath*>				m_PathsList			;	//	motion paths used in the scene
	vector<cParticleSystem*>	m_ParticleSystems	;	//	pointers to particles systems
	vector<sPathLink>			m_PathLinksList		;	//	list of links betwwen paths and points { doest need to be deallocated )
	vector<sPointLink>			m_PointLinksList	;	//	list of point links
	c3DMenu*					m_pMenu				;	//	the 3d menu in the scene, only one instance can exist
	cMarquee*					m_pMarquee			;	//	some associated text with the scene
	cMusic*						m_pMusic			;	//	music for the scene ( if there is one )
	cViewer*					m_pViewer			;	//	input of viewer.. mouse or head tracker
	cFog*						m_pFog				;	//	pointer to fog
			
	//	those are safe and no need for destruction for them..they remain on the stack
	vector<point3d*>	m_VectorsList				;	//	the point3dz which can controlled by the scene
	cCamera				m_Camera					;	//	the scene camera (only one camera for now)
	int					m_FrameRate					;	//	maximum frame rate allowed in the scene, will be 60 by default
	sViewPort			m_ViewPort					;	//	view port for the scene
	point3d				m_MinPoint					;	//	low bound x,y,z values in the world
	point3d				m_MaxPoint					;	//	upper bound x,y,z values in the world
	color3				m_BackColor					;	//	background color, default is black
	
	
	bool				m_bShowFps					;	//	flag to show/hide frame rate
	bool				m_bKeyboardControl			;	//	flag for keyboard control of the camera
	char				m_FileName[256]				;	//	name of the file describing this scene
	char				m_NextSceneFilename[256]	;	//	file name for the next scene


	//	different types of objects
	//	this can be used as an optimization
	vector		<cRigidBody*>	m_RigidBodies	;	//	rigid bodies (draw and update)
	vector		<cRigidBody*>	m_BoundedObjects;	//	rigid bodies (keep in the scene)

	//	viewer response object
	cSecondOrder<point3d>		m_HeadResponse;
	cSecondOrder<point3d>		m_KeysResponse;

	//	the scene loader should be able to access everything here
	friend class cSceneLoader;
	friend class c3DScene;


public:

	//	 constructor always need a file name
	cSceneData(char* FileName);

	//	destructor should destroy all objects and lights
	virtual ~cSceneData()
	{	Kill();	}

	//	Release memory used by the scene
	void Kill();

	//	load a scene description from a text file
	//	return -1 on failure
	int Create(c3DScene* pScenePlayer);

};
/*-------------end of class c3DScene declaration-------------------*/
#endif